#!/bin/sh

PATH=/bin:/sbin:/usr/bin:/usr/sbin

MTD_INFO=`grep "\"Info\"" /proc/mtd | awk -F: '{ print $1 }' | awk -Fd '{ print "/dev/mtdblock"$2 }'`
MODEL=`dd if=${MTD_INFO} of=/dev/stdout bs=16 skip=1 count=1 | sed "s/$/<sep>/g" | sed "s/<sep>//" | sed "s/ //g"`
SH_MODEL=${MODEL#"PRS-"}


W_LOG=/tmp/Data/script.log
W_DIR=/tmp/Rootfs
SD_IMG=/Data/test/new_opt.${SH_MODEL}.img
SD_MD5=/Data/test/new_opt.${SH_MODEL}.md5
# PARTITION=/dev/mtdblock9
PARTITION=$(grep \"Rootfs\" /proc/mtd | awk -F: '{ printf $1 }' | sed 's/mtd/\/dev\/mtdblock/')

clean_exit()
{
   rm -rf /tmp/change_update.sh
   /usr/local/sony/bin/mtdmount -t vfat -o remount,rw,shortname=winnt,iocharset=utf8 Data /Data
   cp -f $W_LOG /Data
   /usr/local/sony/bin/mtdmount -t vfat -o remount,ro,shortname=winnt,iocharset=utf8 Data /Data
   sync
   sync
   sync
   # reboot
   exit 16
}

bad_exit()
{
   /usr/local/sony/bin/nblconfig -ksel recovery
   rm -rf /tmp/change_update.sh
   /usr/local/sony/bin/mtdmount -t vfat -o remount,rw,shortname=winnt,iocharset=utf8 Data /Data
   cp -f $W_LOG /Data
   /usr/local/sony/bin/mtdmount -t vfat -o remount,ro,shortname=winnt,iocharset=utf8 Data /Data
   sync
   sync
   sync
   sleep 300
   # reboot
   exit 0
}

# /usr/local/sony/bin/nblconfig -ksel normal

# /bin/mount -o remount -t tmpfs -o size=51m /dev/shm /tmp

mkdir -p /tmp/Data

grep \/Data /proc/mounts
if [ $? -ne 0 ];
then
   /usr/local/sony/bin/mtdmount -t vfat -o ro,shortname=winnt,iocharset=utf8 Data /Data
fi

echo Starting $0 for $MODEL > $W_LOG

for file in /proc/filesystems /proc/mtd /proc/mounts
do
   echo "-----------------------------" >> $W_LOG
   echo $file >> $W_LOG
   cat $file >> $W_LOG
   echo "-----------------------------" >> $W_LOG
done

echo "df -k before remount" >> $W_LOG
echo "-----------------------------------" >> $W_LOG
/bin/df -k >> $W_LOG
echo "-----------------------------------" >> $W_LOG

echo "/usr/local/sony/bin/nblconfig -dump" >> $W_LOG
echo "-----------------------------------" >> $W_LOG
/usr/local/sony/bin/nblconfig -dump >> $W_LOG
echo "-----------------------------------" >> $W_LOG

# echo "-----------------------------------" >> $W_LOG
# echo "/usr/local/sony/bin/mkdiagms" >> $W_LOG
# /usr/local/sony/bin/mkdiagms >> $W_LOG
# echo "   result $?"
# echo "-----------------------------------" >> $W_LOG

# Flashing the FS

echo -n "Begin flashing -> " >> $W_LOG
$date >> $W_LOG

if [ -f $SD_IMG -a -f $SD_MD5 ]
then
   echo $SD_IMG found >> $W_LOG
   echo $SD_MD5 found >> $W_LOG
else
   ls -l $SD_IMG >> $W_LOG 2>&1
   ls -l $SD_MD5 >> $W_LOG 2>&1

   clean_exit
fi

REAL_MD5=`/usr/bin/md5sum $SD_IMG | cut -f1 -d\  `
TEST_MD5=`cut -f1 -d\  $SD_MD5 `

if [ $REAL_MD5 = $TEST_MD5 ]
then
   echo "Checksum $REAL_MD5 correct" >> $W_LOG
else
   echo "MD5 check failure [$REAL_MD5 != $TEST_MD5]" >> $W_LOG

   clean_exit
fi

echo "Checking cramfs image size ...." >> $W_LOG

# Max Image Size
OPT_IMG_SIZE=$(printf "%d\n" 0x`grep \"Rootfs\" /proc/mtd | awk '{ printf $2 }'`)
# NEW_IMG_SIZE=`$du -b $SD_IMG | $cut -f 1`
NEW_IMG_SIZE=`ls -la $SD_IMG | awk '{ print $5 }'`

echo "Partition size:    $OPT_IMG_SIZE " >> $W_LOG
echo "Cramfs image size: $NEW_IMG_SIZE " >> $W_LOG

if [ $NEW_IMG_SIZE -gt $OPT_IMG_SIZE ]; then
   echo "Rootfs image size is too big!" >> $W_LOG

   clean_exit
fi

echo "Image size is OK" >> $W_LOG

# Flashing !!!

echo "/bin/dd  if=${SD_IMG} of=${PARTITION} bs=1024 conv=notrunc" >> $W_LOG
time /bin/dd if=${SD_IMG} of=${PARTITION} bs=1024 conv=notrunc >> $W_LOG 2>&1

echo "Checking flashed partition against the image..." >> $W_LOG
# echo /usr/local/sony/bin/nblsdm cmp -i $SD_IMG Rootfs >> $W_LOG
mkdir -p $W_DIR
mount $PARTITION $W_DIR >> $W_LOG 2>&1

if [ $? != 0 ]
then
   echo mount failed >> $W_LOG
   bad_exit
fi

echo "df -k after remount" >> $W_LOG
echo "-----------------------------------" >> $W_LOG
/bin/df -k >> $W_LOG
echo "-----------------------------------" >> $W_LOG

if [ ! -f $W_DIR/opt/sony/ebook/application/FskDocumentViewerAdobe.so ]
then
   echo filesystem corrupt >> $W_LOG
   bad_exit
fi

echo "Remove the files from Data..." >> $W_LOG

/usr/local/sony/bin/mtdmount -t vfat -o remount,rw,shortname=winnt,iocharset=utf8 Data /Data
rm -rf /Data/test

echo "Reflashing was successful. Rebooting..." >> $W_LOG

clean_exit
